﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  07/11/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Web.Mvc
{
    using System.IO;
    using System.Web;
    using System.Web.Optimization;

    public static class WebViewPageExtensions
    {
        /// <summary>
        /// Renders any view specific script bundle for the current page.
        /// </summary>
        /// <param name="page">The page to render scripts for.</param>
        /// <returns>A HTML string containing the link tag or tags for the bundle.</returns>
        public static IHtmlString RenderViewScriptBundle(this WebViewPage page)
        {
            return page.RenderViewScriptBundle(false);
        }

        /// <summary>
        /// Renders any view specific script bundle for the current page.
        /// </summary>
        /// <param name="page">The page to render scripts for.</param>
        /// <param name="force_bundle">true to force render bundle; otherwise, false.</param>
        /// <returns>A HTML string containing the link tag or tags for the bundle.</returns>
        public static IHtmlString RenderViewScriptBundle(this WebViewPage page, bool force_bundle)
        {
            //get the view path:
            string viewPath = ((BuildManagerCompiledView)page.ViewContext.View).ViewPath;

            //get the controller:
            string controller = (new DirectoryInfo(Path.GetDirectoryName(viewPath)).Name);

            //get the view name:
            string view = Path.GetFileNameWithoutExtension(viewPath);

            //create the script bundle virtual path:
            string bundleVirtualPath = "~/bundles/assets/" + controller + "/" + view;

            //locate any available bundle:
            string bundle = BundleTable.Bundles.ResolveBundleUrl(bundleVirtualPath, true);

            if (bundle != null) //if bundle exists
                return Scripts.Render(force_bundle ? bundle : bundleVirtualPath);

            return MvcHtmlString.Empty;
        }

        /// <summary>
        /// Renders any view specific style bundle for the current page.
        /// </summary>
        /// <param name="page">The page to render styles for.</param>
        /// <returns>A HTML string containing the link tag or tags for the bundle.</returns>
        public static IHtmlString RenderViewStyleBundle(this WebViewPage page)
        {
            return page.RenderViewStyleBundle(false);
        }

        /// <summary>
        /// Renders any view specific style bundle for the current page.
        /// </summary>
        /// <param name="page">The page to render styles for.</param>
        /// <param name="force_bundle">true to force render bundle; otherwise, false.</param>
        /// <returns>A HTML string containing the link tag or tags for the bundle.</returns>
        public static IHtmlString RenderViewStyleBundle(this WebViewPage page, bool force_bundle)
        {
            //get the view path:
            string viewPath = ((BuildManagerCompiledView)page.ViewContext.View).ViewPath;

            //get the controller:
            string controller = (new DirectoryInfo(Path.GetDirectoryName(viewPath)).Name);

            //get the view name:
            string view = Path.GetFileNameWithoutExtension(viewPath);

            //create the style bundle virtual path:
            string bundleVirtualPath = "~/Content/Assets/" + controller + "/" + view;

            //locate any available bundle:
            string bundle = BundleTable.Bundles.ResolveBundleUrl(bundleVirtualPath, true);

            if (bundle != null) //if bundle exists
                return Styles.Render(force_bundle ? bundle : bundleVirtualPath);

            return MvcHtmlString.Empty;
        }
    }
}
