﻿//---------------------------------------------------------------------------- 
//
//  Copyright (C) CSharp Labs.  All rights reserved.
// 
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
// 
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
// 
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
// 
// History
//  07/11/13    Created 
//
//---------------------------------------------------------------------------

namespace System.Web.Optimization
{
    using System.IO;
    using System.Web.Hosting;

    public static class BundleCollectionExtensions
    {
        /// <summary>
        /// Registers script bundles for views.
        /// </summary>
        /// <param name="bundle">The BundleCollection to add script bundles to.</param>
        public static void RegisterViewScriptBundles(this BundleCollection bundle)
        {
            bundle.RegisterViewBundles(".js");
        }

        /// <summary>
        /// Registers style bundles for views.
        /// </summary>
        /// <param name="bundle">The BundleCollection to add style bundles to.</param>
        public static void RegisterViewStyleBundles(this BundleCollection bundle)
        {
            bundle.RegisterViewBundles(".css");
        }

        /// <summary>
        /// Registers script or style bundles for views.
        /// </summary>
        /// <param name="bundle">The BundleCollection to add script or style bundles to.</param>
        /// <param name="extension">The file extension to use to add files.</param>
        /// <exception cref="ArgumentException"></exception>
        private static void RegisterViewBundles(this BundleCollection bundle, string extension)
        {
            //defines a path to the assets folder:
            const string virtualPath = "~/Content/Assets";

            bool java;
            switch (extension.ToLower())
            {
                case ".js": //JavaScript file
                    java = true;
                    break;
                case ".css": //Cascading Style Sheet file
                    java = false;
                    break;
                default:
                    throw new ArgumentException("Invalid extension specified: '" + extension + "'.");
            }

            string path = HostingEnvironment.MapPath(virtualPath); //get file path of asset folder

            if (Directory.Exists(path)) //if asset folder exists
            {
                string viewsDirectory = HostingEnvironment.MapPath("~/Views"); //get views directory
                
                foreach (string viewPath in Directory.GetDirectories(path)) //get all asset controller file paths
                {
                    string controllerName = (new DirectoryInfo(viewPath)).Name; //get the controller name
                    string controllerDirectory = Path.Combine(viewsDirectory, controllerName); //create the controller directory containing views

                    if (Directory.Exists(controllerDirectory)) //if controller directory exists
                    {
                        //validate each JavaScript or CSS file looking for a corresponding view
                        foreach (string filePath in Directory.GetFiles(viewPath, "*" + extension)) //get all java or css files for the view
                        {
                            string viewFileName = Path.GetFileName(filePath); //get view file name
                            string viewName = Path.GetFileNameWithoutExtension(viewFileName); //get view name
                            string viewFilePath = Path.Combine(controllerDirectory, viewName); //get view file path

                            if (File.Exists(viewFilePath + ".cshtml") || File.Exists(viewFilePath + ".aspx")) //if view file exists
                            {
                                string actualVirtualPropPath = virtualPath + "/" + controllerName + "/" + viewFileName; //setup virtual path to asset

                                if (java) //if java, register ScriptBundle
                                    bundle.Add(new ScriptBundle("~/bundles/assets/" + controllerName + "/" + viewName).Include(actualVirtualPropPath));
                                else //otherwise register StyleBundle
                                    bundle.Add(new StyleBundle("~/Content/Assets/" + controllerName + "/" + viewName).Include(actualVirtualPropPath));
                            }
                        }
                    }
                }
            }
        }
    }
}
